==> gmssl-jsse-provider-0.1.4-20190730.152528-2-sources.jar: org/bouncycastle/jsse/provider/XDJAJsseProvider.java <==
package org.bouncycastle.jsse.provider;

import org.bouncycastle.tls.crypto.impl.bc.BcTlsCryptoProvider;
import org.bouncycastle.tls.crypto.impl.bc.GMSSLCryptoType;
import org.bouncycastle.util.Strings;

import java.security.GeneralSecurityException;
import java.security.NoSuchAlgorithmException;
import java.security.Provider;
import java.util.*;

@SuppressWarnings("serial")
public class XDJAJsseProvider
        extends Provider {
    public static final String PROVIDER_NAME = "XDJAJSSE";

    private static final double PROVIDER_VERSION = 1.0004;
    private static final String PROVIDER_INFO = "XDJA Bouncy Castle JSSE Provider Version 1.0.4";

    private Map<String, BcJsseService> serviceMap = new HashMap<>();
    private Map<String, EngineCreator> creatorMap = new HashMap<>();

    private final boolean isInFipsMode;

    public XDJAJsseProvider() {
        super(PROVIDER_NAME, PROVIDER_VERSION, PROVIDER_INFO);

        this.isInFipsMode = configure(new BcTlsCryptoProvider());
    }

    private boolean configure(final BcTlsCryptoProvider baseCryptoProvider) {
        final boolean fipsMode = false;
        //添加算法实现
        // TODO[jsse]: should X.509 be an alias.
        addAlgorithmImplementation("KeyManagerFactory.X.509", "org.bouncycastle.jsse.provider.KeyManagerFactory", new EngineCreator() {
            @Override
            public Object createInstance(Object constructorParameter) {
                return new ProvKeyManagerFactorySpi();
            }
        });
        addAlias("Alg.Alias.KeyManagerFactory.X509", "X.509");
        addAlias("Alg.Alias.KeyManagerFactory.PKIX", "X.509");

        addAlgorithmImplementation("TrustManagerFactory.PKIX", "org.bouncycastle.jsse.provider.TrustManagerFactory", new EngineCreator() {
            @Override
            public Object createInstance(Object constructorParameter) {
                return new ProvTrustManagerFactorySpi(baseCryptoProvider.getPkixProvider());
            }
        });

        addAlias("Alg.Alias.TrustManagerFactory.X.509", "PKIX");
        addAlias("Alg.Alias.TrustManagerFactory.X509", "PKIX");

        addAlgorithmImplementation("SSLContext.TLS", "org.bouncycastle.jsse.provider.SSLContext.TLS",
                new EngineCreator() {
                    @Override
                    public Object createInstance(Object constructorParameter) {
                        return new ProvSSLContextSpi(fipsMode, baseCryptoProvider, null);
                    }
                });
        addAlgorithmImplementation("SSLContext.TLSV1", "org.bouncycastle.jsse.provider.SSLContext.TLSv1",
                new EngineCreator() {
                    @Override
                    public Object createInstance(Object constructorParameter) {
                        return new ProvSSLContextSpi(fipsMode, baseCryptoProvider, new String[]{"TLSv1", "TLSv1.1", "TLSv1.2"});
                    }
                });
        addAlgorithmImplementation("SSLContext.TLSV1.1", "org.bouncycastle.jsse.provider.SSLContext.TLSv1_1",
                new EngineCreator() {
                    @Override
                    public Object createInstance(Object constructorParameter) {
                        return new ProvSSLContextSpi(fipsMode, baseCryptoProvider, new String[]{"TLSv1.1", "TLSv1.2"});
                    }
                });
        addAlgorithmImplementation("SSLContext.TLSV1.2", "org.bouncycastle.jsse.provider.SSLContext.TLSv1_2",
                new EngineCreator() {
                    @Override
                    public Object createInstance(Object constructorParameter) {
                        return new ProvSSLContextSpi(fipsMode, baseCryptoProvider, new String[]{"TLSv1.2"});
                    }
                });
        addAlgorithmImplementation("SSLContext.DEFAULT", "org.bouncycastle.jsse.provider.SSLContext.Default",
                new EngineCreator() {
                    @Override
                    public Object createInstance(Object constructorParameter) {
                        try {
                            ProvSSLContextSpi defaultSSLContextSpi = new ProvSSLContextSpi(fipsMode, baseCryptoProvider, null);
                            defaultSSLContextSpi.engineInit(null, null, null);
                            return defaultSSLContextSpi;
                        } catch (GeneralSecurityException e) {
                            // TODO[jsse] Log this exception
                            return null;
                        }
                    }
                });

        //GMSSL SUPPORT: add gmssl v1.1
        addAlgorithmImplementation("SSLContext.GMSSLV1.1", "org.bouncycastle.jsse.provider.SSLContext.GMSSLv1_1",
                new EngineCreator() {
                    @Override
                    public Object createInstance(Object constructorParameter) {
                        baseCryptoProvider.setType(GMSSLCryptoType.BC);
                        return new ProvSSLContextSpi(fipsMode, baseCryptoProvider, new String[]{"GMSSLv1.1"});
                    }
                });

        //GMSSL SUPPORT: add sdf pcie gmssl v1.1
        addAlgorithmImplementation("SSLContext.GMSSLSDFPCIEV1.1", "org.bouncycastle.jsse.provider.SSLContext.GMSSLSDFPCIEv1_1",
                new EngineCreator() {
                    @Override
                    public Object createInstance(Object constructorParameter) {
                        baseCryptoProvider.setType(GMSSLCryptoType.SDF_PCIE);
                        return new ProvSSLContextSpi(fipsMode, baseCryptoProvider, new String[]{"GMSSLv1.1"});
                    }
                });

        //GMSSL SUPPORT: add sdf yunhsm gmssl v1.1
        addAlgorithmImplementation("SSLContext.GMSSLSDFYUNHSMV1.1", "org.bouncycastle.jsse.provider.SSLContext.GMSSLSDFYUNHSMv1_1",
                new EngineCreator() {
                    @Override
                    public Object createInstance(Object constructorParameter) {
                        baseCryptoProvider.setType(GMSSLCryptoType.SDF_YUNHSM);
                        return new ProvSSLContextSpi(fipsMode, baseCryptoProvider, new String[]{"GMSSLv1.1"});
                    }
                });

        return fipsMode;
    }

    void addAttribute(String key, String attributeName, String attributeValue) {
        String attributeKey = key + " " + attributeName;
        if (containsKey(attributeKey)) {
            throw new IllegalStateException("duplicate provider attribute key (" + attributeKey + ") found");
        }

        put(attributeKey, attributeValue);
    }

    void addAlgorithmImplementation(String key, String className, EngineCreator creator) {
        if (containsKey(key)) {
            throw new IllegalStateException("duplicate provider key (" + key + ") found");
        }

        addAttribute(key, "ImplementedIn", "Software");

        put(key, className);
        creatorMap.put(className, creator);
    }

    void addAlias(String key, String value) {
        if (containsKey(key)) {
            throw new IllegalStateException("duplicate provider key (" + key + ") found");
        }

        put(key, value);
    }

    @Override
    public synchronized final Service getService(String type, String algorithm) {
        String upperCaseAlgName = Strings.toUpperCase(algorithm);

        BcJsseService service = serviceMap.get(type + "." + upperCaseAlgName);

        if (service == null) {
            String aliasString = "Alg.Alias." + type + ".";
            String realName = (String) this.get(aliasString + upperCaseAlgName);

            if (realName == null) {
                realName = upperCaseAlgName;
            }

            String className = (String) this.get(type + "." + realName);

            if (className == null) {
                return null;
            }

            String attributeKeyStart = type + "." + upperCaseAlgName + " ";

            List<String> aliases = new ArrayList<String>();
            Map<String, String> attributes = new HashMap<String, String>();

            for (Object key : this.keySet()) {
                String sKey = (String) key;
                if (sKey.startsWith(aliasString)) {
                    if (this.get(key).equals(algorithm)) {
                        aliases.add(sKey.substring(aliasString.length()));
                    }
                }
                if (sKey.startsWith(attributeKeyStart)) {
                    attributes.put(sKey.substring(attributeKeyStart.length()), (String) this.get(sKey));
                }
            }

            service = new BcJsseService(this, type, upperCaseAlgName, className, aliases, getAttributeMap(attributes), creatorMap.get(className));

            serviceMap.put(type + "." + upperCaseAlgName, service);
        }

        return service;
    }

    @Override
    public synchronized final Set<Service> getServices() {
        Set<Service> serviceSet = super.getServices();
        Set<Service> bcServiceSet = new HashSet<Service>();

        for (Service service : serviceSet) {
            bcServiceSet.add(getService(service.getType(), service.getAlgorithm()));
        }

        return bcServiceSet;
    }

    private static final Map<Map<String, String>, Map<String, String>> attributeMaps = new HashMap<Map<String, String>, Map<String, String>>();

    private static Map<String, String> getAttributeMap(Map<String, String> attributeMap) {
        Map<String, String> attrMap = attributeMaps.get(attributeMap);
        if (attrMap != null) {
            return attrMap;
        }

        attributeMaps.put(attributeMap, attributeMap);

        return attributeMap;
    }

    public boolean isFipsMode() {
        return isInFipsMode;
    }

    private static class BcJsseService
            extends Service {
        private final EngineCreator creator;

        /**
         * Construct a new service.
         *
         * @param provider   the provider that offers this service
         * @param type       the type of this service
         * @param algorithm  the algorithm name
         * @param className  the name of the class implementing this service
         * @param aliases    List of aliases or null if algorithm has no aliases
         * @param attributes Map of attributes or null if this implementation
         *                   has no attributes
         * @throws NullPointerException if provider, type, algorithm, or
         *                              className is null
         */
        public BcJsseService(Provider provider, String type, String algorithm, String className, List<String> aliases, Map<String, String> attributes, EngineCreator creator) {
            super(provider, type, algorithm, className, aliases, attributes);
            this.creator = creator;
        }

        @Override
        public Object newInstance(Object constructorParameter)
                throws NoSuchAlgorithmException {
            try {
                Object instance = creator.createInstance(constructorParameter);

                if (instance == null) {
                    throw new NoSuchAlgorithmException("No such algorithm in FIPS approved mode: " + getAlgorithm());
                }

                return instance;
            } catch (NoSuchAlgorithmException e) {
                throw e;
            } catch (Exception e) {
                throw new NoSuchAlgorithmException("Unable to invoke creator for " + getAlgorithm() + ": " + e.getMessage(), e);
            }
        }
    }

//    private static final class FipsCapabilities
//        extends TlsCryptoCapabilities
//    {
//        public FipsCapabilities()
//        {
//            super(new int[]{ NamedGroup.secp256r1, NamedGroup.secp384r1 });
//        }
//    }
}

==> gmssl-jsse-provider-1.0.1-20191211.090138-1-sources.jar: org/bouncycastle/jsse/provider/XDJAJsseProvider.java <==
package org.bouncycastle.jsse.provider;

import org.bouncycastle.tls.crypto.impl.bc.BcTlsCryptoProvider;
import org.bouncycastle.tls.crypto.impl.bc.GMSSLCryptoType;
import org.bouncycastle.util.Strings;

import java.security.GeneralSecurityException;
import java.security.NoSuchAlgorithmException;
import java.security.Provider;
import java.util.*;

@SuppressWarnings("serial")
public class XDJAJsseProvider extends Provider {
    public static final String PROVIDER_NAME = "XDJAJSSE";

    private static final double PROVIDER_VERSION = 1.0004;
    private static final String PROVIDER_INFO = "XDJA Bouncy Castle JSSE Provider Version 1.0.4";

    private Map<String, BcJsseService> serviceMap = new HashMap<>();
    private Map<String, EngineCreator> creatorMap = new HashMap<>();

    private final boolean isInFipsMode;

    public XDJAJsseProvider() {
        super(PROVIDER_NAME, PROVIDER_VERSION, PROVIDER_INFO);

        this.isInFipsMode = configure(new BcTlsCryptoProvider());
    }

    private boolean configure(final BcTlsCryptoProvider baseCryptoProvider) {
        final boolean fipsMode = false;
        //添加算法实现
        // TODO[jsse]: should X.509 be an alias.
        addAlgorithmImplementation("KeyManagerFactory.X.509", "org.bouncycastle.jsse.provider.KeyManagerFactory", new EngineCreator() {
            @Override
            public Object createInstance(Object constructorParameter) {
                return new ProvKeyManagerFactorySpi();
            }
        });
        addAlias("Alg.Alias.KeyManagerFactory.X509", "X.509");
        addAlias("Alg.Alias.KeyManagerFactory.PKIX", "X.509");

        addAlgorithmImplementation("TrustManagerFactory.PKIX", "org.bouncycastle.jsse.provider.TrustManagerFactory", new EngineCreator() {
            @Override
            public Object createInstance(Object constructorParameter) {
                return new ProvTrustManagerFactorySpi(baseCryptoProvider.getPkixProvider());
            }
        });

        addAlias("Alg.Alias.TrustManagerFactory.X.509", "PKIX");
        addAlias("Alg.Alias.TrustManagerFactory.X509", "PKIX");

        addAlgorithmImplementation("SSLContext.TLS", "org.bouncycastle.jsse.provider.SSLContext.TLS",
                new EngineCreator() {
                    @Override
                    public Object createInstance(Object constructorParameter) {
                        return new ProvSSLContextSpi(fipsMode, baseCryptoProvider, null);
                    }
                });
        addAlgorithmImplementation("SSLContext.TLSV1", "org.bouncycastle.jsse.provider.SSLContext.TLSv1",
                new EngineCreator() {
                    @Override
                    public Object createInstance(Object constructorParameter) {
                        return new ProvSSLContextSpi(fipsMode, baseCryptoProvider, new String[]{"TLSv1", "TLSv1.1", "TLSv1.2"});
                    }
                });
        addAlgorithmImplementation("SSLContext.TLSV1.1", "org.bouncycastle.jsse.provider.SSLContext.TLSv1_1",
                new EngineCreator() {
                    @Override
                    public Object createInstance(Object constructorParameter) {
                        return new ProvSSLContextSpi(fipsMode, baseCryptoProvider, new String[]{"TLSv1.1", "TLSv1.2"});
                    }
                });
        addAlgorithmImplementation("SSLContext.TLSV1.2", "org.bouncycastle.jsse.provider.SSLContext.TLSv1_2",
                new EngineCreator() {
                    @Override
                    public Object createInstance(Object constructorParameter) {
                        return new ProvSSLContextSpi(fipsMode, baseCryptoProvider, new String[]{"TLSv1.2"});
                    }
                });
        addAlgorithmImplementation("SSLContext.DEFAULT", "org.bouncycastle.jsse.provider.SSLContext.Default",
                new EngineCreator() {
                    @Override
                    public Object createInstance(Object constructorParameter) {
                        try {
                            ProvSSLContextSpi defaultSSLContextSpi = new ProvSSLContextSpi(fipsMode, baseCryptoProvider, null);
                            defaultSSLContextSpi.engineInit(null, null, null);
                            return defaultSSLContextSpi;
                        } catch (GeneralSecurityException e) {
                            // TODO[jsse] Log this exception
                            return null;
                        }
                    }
                });

        //GMSSL SUPPORT: add gmssl v1.1
        addAlgorithmImplementation("SSLContext.GMSSLV1.1", "org.bouncycastle.jsse.provider.SSLContext.GMSSLv1_1",
                new EngineCreator() {
                    @Override
                    public Object createInstance(Object constructorParameter) {
                        baseCryptoProvider.setType(GMSSLCryptoType.BC);
                        return new ProvSSLContextSpi(fipsMode, baseCryptoProvider, new String[]{"GMSSLv1.1"});
                    }
                });

        //GMSSL SUPPORT: add sdf pcie gmssl v1.1
        addAlgorithmImplementation("SSLContext.GMSSLXDJASDFPCIEV1.1", "org.bouncycastle.jsse.provider.SSLContext.GMSSLXDJASDFPCIEv1_1",
                new EngineCreator() {
                    @Override
                    public Object createInstance(Object constructorParameter) {
                        baseCryptoProvider.setType(GMSSLCryptoType.XDJA_SDF_PCIE);
                        return new ProvSSLContextSpi(fipsMode, baseCryptoProvider, new String[]{"GMSSLv1.1"});
                    }
                });

        //GMSSL SUPPORT: add sdf yunhsm gmssl v1.1
        addAlgorithmImplementation("SSLContext.GMSSLXDJASDFYUNHSMV1.1", "org.bouncycastle.jsse.provider.SSLContext.GMSSLXDJASDFYUNHSMv1_1",
                new EngineCreator() {
                    @Override
                    public Object createInstance(Object constructorParameter) {
                        baseCryptoProvider.setType(GMSSLCryptoType.XDJA_SDF_YUNHSM);
                        return new ProvSSLContextSpi(fipsMode, baseCryptoProvider, new String[]{"GMSSLv1.1"});
                    }
                });

        //GMSSL SUPPORT: add sdf pcie gmssl v1.1 with sm2 but sm3 sm4 is bc
        addAlgorithmImplementation("SSLContext.GMSSLSDFPCIEV1.1", "org.bouncycastle.jsse.provider.SSLContext.GMSSLSDFPCIEv1_1",
                new EngineCreator() {
                    @Override
                    public Object createInstance(Object constructorParameter) {
                        baseCryptoProvider.setType(GMSSLCryptoType.SDF_PCIE);
                        return new ProvSSLContextSpi(fipsMode, baseCryptoProvider, new String[]{"GMSSLv1.1"});
                    }
                });

        //GMSSL SUPPORT: add sdf yunhsm gmssl v1.1 with sm2 but sm3 sm4 is bc
        addAlgorithmImplementation("SSLContext.GMSSLSDFYUNHSMV1.1", "org.bouncycastle.jsse.provider.SSLContext.GMSSLSDFYUNHSMv1_1",
                new EngineCreator() {
                    @Override
                    public Object createInstance(Object constructorParameter) {
                        baseCryptoProvider.setType(GMSSLCryptoType.SDF_YUNHSM);
                        return new ProvSSLContextSpi(fipsMode, baseCryptoProvider, new String[]{"GMSSLv1.1"});
                    }
                });

        return fipsMode;
    }

    void addAttribute(String key, String attributeName, String attributeValue) {
        String attributeKey = key + " " + attributeName;
        if (containsKey(attributeKey)) {
            throw new IllegalStateException("duplicate provider attribute key (" + attributeKey + ") found");
        }

        put(attributeKey, attributeValue);
    }

    void addAlgorithmImplementation(String key, String className, EngineCreator creator) {
        if (containsKey(key)) {
            throw new IllegalStateException("duplicate provider key (" + key + ") found");
        }

        addAttribute(key, "ImplementedIn", "Software");

        put(key, className);
        creatorMap.put(className, creator);
    }

    void addAlias(String key, String value) {
        if (containsKey(key)) {
            throw new IllegalStateException("duplicate provider key (" + key + ") found");
        }

        put(key, value);
    }

    @Override
    public synchronized final Service getService(String type, String algorithm) {
        String upperCaseAlgName = Strings.toUpperCase(algorithm);

        BcJsseService service = serviceMap.get(type + "." + upperCaseAlgName);

        if (service == null) {
            String aliasString = "Alg.Alias." + type + ".";
            String realName = (String) this.get(aliasString + upperCaseAlgName);

            if (realName == null) {
                realName = upperCaseAlgName;
            }

            String className = (String) this.get(type + "." + realName);

            if (className == null) {
                return null;
            }

            String attributeKeyStart = type + "." + upperCaseAlgName + " ";

            List<String> aliases = new ArrayList<String>();
            Map<String, String> attributes = new HashMap<String, String>();

            for (Object key : this.keySet()) {
                String sKey = (String) key;
                if (sKey.startsWith(aliasString)) {
                    if (this.get(key).equals(algorithm)) {
                        aliases.add(sKey.substring(aliasString.length()));
                    }
                }
                if (sKey.startsWith(attributeKeyStart)) {
                    attributes.put(sKey.substring(attributeKeyStart.length()), (String) this.get(sKey));
                }
            }

            service = new BcJsseService(this, type, upperCaseAlgName, className, aliases, getAttributeMap(attributes), creatorMap.get(className));

            serviceMap.put(type + "." + upperCaseAlgName, service);
        }

        return service;
    }

    @Override
    public synchronized final Set<Service> getServices() {
        Set<Service> serviceSet = super.getServices();
        Set<Service> bcServiceSet = new HashSet<Service>();

        for (Service service : serviceSet) {
            bcServiceSet.add(getService(service.getType(), service.getAlgorithm()));
        }

        return bcServiceSet;
    }

    private static final Map<Map<String, String>, Map<String, String>> attributeMaps = new HashMap<Map<String, String>, Map<String, String>>();

    private static Map<String, String> getAttributeMap(Map<String, String> attributeMap) {
        Map<String, String> attrMap = attributeMaps.get(attributeMap);
        if (attrMap != null) {
            return attrMap;
        }

        attributeMaps.put(attributeMap, attributeMap);

        return attributeMap;
    }

    public boolean isFipsMode() {
        return isInFipsMode;
    }

    private static class BcJsseService
            extends Service {
        private final EngineCreator creator;

        /**
         * Construct a new service.
         *
         * @param provider   the provider that offers this service
         * @param type       the type of this service
         * @param algorithm  the algorithm name
         * @param className  the name of the class implementing this service
         * @param aliases    List of aliases or null if algorithm has no aliases
         * @param attributes Map of attributes or null if this implementation
         *                   has no attributes
         * @throws NullPointerException if provider, type, algorithm, or
         *                              className is null
         */
        public BcJsseService(Provider provider, String type, String algorithm, String className, List<String> aliases, Map<String, String> attributes, EngineCreator creator) {
            super(provider, type, algorithm, className, aliases, attributes);
            this.creator = creator;
        }

        @Override
        public Object newInstance(Object constructorParameter)
                throws NoSuchAlgorithmException {
            try {
                Object instance = creator.createInstance(constructorParameter);

                if (instance == null) {
                    throw new NoSuchAlgorithmException("No such algorithm in FIPS approved mode: " + getAlgorithm());
                }

                return instance;
            } catch (NoSuchAlgorithmException e) {
                throw e;
            } catch (Exception e) {
                throw new NoSuchAlgorithmException("Unable to invoke creator for " + getAlgorithm() + ": " + e.getMessage(), e);
            }
        }
    }

//    private static final class FipsCapabilities
//        extends TlsCryptoCapabilities
//    {
//        public FipsCapabilities()
//        {
//            super(new int[]{ NamedGroup.secp256r1, NamedGroup.secp384r1 });
//        }
//    }
}
